@@ -19,6 +19,7 @@
 import io
 import sys
 import six
+import types
 from six import StringIO
 from io import BytesIO
 from lxml import etree
@@ -29,7 +30,13 @@ from lxml import etree
 
 from ncclient import NCClientError
 
-parser = etree.XMLParser(recover=True)
+parser = etree.XMLParser(recover=False)
+huge_parser = etree.XMLParser(recover=False, huge_tree=True)
+
+
+def _get_parser(huge_tree=False):
+    return huge_parser if huge_tree else parser
+
 
 class XMLError(NCClientError):
     pass
@@ -38,9 +45,11 @@ class XMLError(NCClientError):
 
 #: Base NETCONF namespace
 BASE_NS_1_0 = "urn:ietf:params:xml:ns:netconf:base:1.0"
-# NXOS_1_0
+#: YANG (RFC 6020/RFC 7950) namespace
+YANG_NS_1_0 = "urn:ietf:params:xml:ns:yang:1"
+#: NXOS_1_0
 NXOS_1_0 = "http://www.cisco.com/nxos:1.0"
-# NXOS_IF
+#: NXOS_IF
 NXOS_IF = "http://www.cisco.com/nxos:1.0:if_manager"
 #: Namespace for Tail-f core data model
 TAILF_AAA_1_1 = "http://tail-f.com/ns/aaa/1.1"
@@ -66,7 +75,14 @@ H3C_ACTION_1_0 = "http://www.h3c.com/netconf/action:1.0"
 NETCONF_MONITORING_NS = "urn:ietf:params:xml:ns:yang:ietf-netconf-monitoring"
 #: Namespace for netconf notifications
 NETCONF_NOTIFICATION_NS = "urn:ietf:params:xml:ns:netconf:notification:1.0"
-#
+#: Namespace for netconf with-defaults (RFC 6243)
+NETCONF_WITH_DEFAULTS_NS = "urn:ietf:params:xml:ns:yang:ietf-netconf-with-defaults"
+#: Namespace for Alcatel-Lucent SR OS Base r13 YANG models
+ALU_CONFIG = "urn:alcatel-lucent.com:sros:ns:yang:conf-r13"
+#: Namespace for Nokia SR OS global operations
+SROS_GLOBAL_OPS_NS = "urn:nokia.com:sros:ns:yang:sr:oper-global"
+
+
 try:
     register_namespace = etree.register_namespace
 except AttributeError:
@@ -101,12 +117,17 @@ def to_xml(ele, encoding="UTF-8", pretty_print=False):
         return xml.decode('UTF-8') if xml.startswith(b'<?xml') \
             else '<?xml version="1.0" encoding="%s"?>%s' % (encoding, xml.decode('UTF-8'))
 
-def to_ele(x):
-    "Convert and return the :class:`~xml.etree.ElementTree.Element` for the XML document *x*. If *x* is already an :class:`~xml.etree.ElementTree.Element` simply returns that."
+
+def to_ele(x, huge_tree=False):
+    """Convert and return the :class:`~xml.etree.ElementTree.Element` for the XML document *x*. If *x* is already an :class:`~xml.etree.ElementTree.Element` simply returns that.
+
+    *huge_tree*: parse XML with very deep trees and very long text content
+    """
     if sys.version < '3':
-        return x if etree.iselement(x) else etree.fromstring(x, parser=parser)
+        return x if etree.iselement(x) else etree.fromstring(x, parser=_get_parser(huge_tree))
     else:
-        return x if etree.iselement(x) else etree.fromstring(x.encode('UTF-8'), parser=parser)
+        return x if etree.iselement(x) else etree.fromstring(x.encode('UTF-8'), parser=_get_parser(huge_tree))
+
 
 def parse_root(raw):
     "Efficiently parses the root element of a *raw* XML document, returning a tuple of its qualified name and attribute dictionary."
@@ -146,20 +167,33 @@ XPATH_NAMESPACES = {
     're':'http://exslt.org/regular-expressions'
 }
 
+
 class NCElement(object):
-    def __init__(self, result, transform_reply):
+    def __init__(self, result, transform_reply, huge_tree=False):
         self.__result = result
         self.__transform_reply = transform_reply
-        self.__doc = self.remove_namespaces(self.__result)
+        self.__huge_tree = huge_tree
+        if isinstance(transform_reply, types.FunctionType):
+            self.__doc = self.__transform_reply(result._root)
+        else:
+            self.__doc = self.remove_namespaces(self.__result)
 
+    def xpath(self, expression, namespaces={}):
+        """Perform XPath navigation on an object
 
-    def xpath(self, expression):
-        """
-            return result for a call to lxml xpath()
-            output will be a list
+        Args:
+            expression: A string representing a compliant XPath
+                expression.
+            namespaces: A dict of caller supplied prefix/xmlns to
+                append to the static dict of XPath namespaces.
+        Returns:
+            A list of 'lxml.etree._Element' should a match on the
+            expression be successful.  Otherwise, an empty list will
+            be returned to the caller.
         """
         self.__expression = expression
         self.__namespaces = XPATH_NAMESPACES
+        self.__namespaces.update(namespaces)
         return self.__doc.xpath(self.__expression, namespaces=self.__namespaces)
 
     def find(self, expression):
@@ -172,10 +206,14 @@ class NCElement(object):
         self.__expression = expression
         return self.__doc.findtext(self.__expression)
 
+    def findall(self, expression):
+        """return result for a call to lxml ElementPath findall()"""
+        self.__expression = expression
+        return self.__doc.findall(self.__expression)
 
     def __str__(self):
         """syntactic sugar for str() - alias to tostring"""
-        if sys.version<'3':
+        if sys.version < '3':
             return self.tostring
         else:
             return self.tostring.decode('UTF-8')
@@ -183,7 +221,7 @@ class NCElement(object):
     @property
     def tostring(self):
         """return a pretty-printed string output for rpc reply"""
-        parser = etree.XMLParser(remove_blank_text=True)
+        parser = etree.XMLParser(remove_blank_text=True, huge_tree=self.__huge_tree)
         outputtree = etree.XML(etree.tostring(self.__doc), parser)
         return etree.tostring(outputtree, pretty_print=True)
 
@@ -195,15 +233,66 @@ class NCElement(object):
     def remove_namespaces(self, rpc_reply):
         """remove xmlns attributes from rpc reply"""
         self.__xslt=self.__transform_reply
-        self.__parser = etree.XMLParser(remove_blank_text=True)
+        self.__parser = etree.XMLParser(remove_blank_text=True, huge_tree=self.__huge_tree)
         self.__xslt_doc = etree.parse(io.BytesIO(self.__xslt), self.__parser)
         self.__transform = etree.XSLT(self.__xslt_doc)
-        self.__root = etree.fromstring(str(self.__transform(etree.parse(StringIO(str(rpc_reply))))))
+        self.__root = etree.fromstring(str(self.__transform(etree.parse(StringIO(str(rpc_reply)),
+                                                                        parser=self.__parser))),
+                                       parser=self.__parser)
         return self.__root
 
+def parent_ns(node):
+    if node.prefix:
+        return node.nsmap[node.prefix]
+    return None
+
+def yang_action(name, attrs):
+    """Instantiate a YANG action element
+
+    Args:
+        name: A string representing the first descendant name of the
+            XML element for the YANG action.
+        attrs: A dict of attributes to apply to the XML element
+            (e.g. namespaces).
+    Returns:
+        A tuple of 'lxml.etree._Element' values.  The first value
+        represents the top-level YANG action element and the second
+        represents the caller supplied initial node.
+    """
+    node = new_ele('action', attrs={'xmlns': YANG_NS_1_0})
+    return (node, sub_ele(node, name, attrs))
+
+
+def replace_namespace(root, old_ns, new_ns):
+    """
+    Substitute old_ns with new_ns for all the xml elements including and below root
+    :param root: top element (root for this change)
+    :param old_ns: old namespace
+    :param new_ns: new namespace
+    :return:
+    """
+    for elem in root.getiterator():
+        # Comments don't have a namespace
+        if elem.tag is not etree.Comment:
+            # handle tag
+            qtag = etree.QName(elem)
+            if qtag.namespace == old_ns:
+                elem.tag = etree.QName(new_ns, qtag.localname)
+
+            # handle attributes
+            attribs_dict = elem.attrib
+            for attr in attribs_dict.keys():
+                qattr = etree.QName(attr)
+                if qattr.namespace == old_ns:
+                    attribs_dict[etree.QName(new_ns, qattr.localname)] = attribs_dict.pop(attr)
+
+
+new_ele_nsmap = lambda tag, nsmap, attrs={}, **extra: etree.Element(qualify(tag), attrs, nsmap, **extra)
 
 new_ele = lambda tag, attrs={}, **extra: etree.Element(qualify(tag), attrs, **extra)
 
 new_ele_ns = lambda tag, ns, attrs={}, **extra: etree.Element(qualify(tag,ns), attrs, **extra)
 
-sub_ele = lambda parent, tag, attrs={}, **extra: etree.SubElement(parent, qualify(tag), attrs, **extra)
+sub_ele = lambda parent, tag, attrs={}, **extra: etree.SubElement(parent, qualify(tag, parent_ns(parent)), attrs, **extra)
+
+sub_ele_ns = lambda parent, tag, ns, attrs={}, **extra: etree.SubElement(parent, qualify(tag, ns), attrs, **extra)
